<?php
namespace SupplyTek\Controllers;

use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use SupplyTek\Utilities\CrudSupplyTek;
use SupplyTek\Utilities\SessionHandler;
use SupplyTek\Utilities\FormatData;
use SupplyTek\Services\NotificationsService;

/**
 * Handles Branch related requests
 * 
 * @author Rômulo Alves Lousada
 */
class BranchController 
{
    protected $crud;
    protected $session;
    protected $format;
    protected $notificationService;

    /**
     * Class constructor
     * 
     * Initialize an object's properties upon creation of the object.
     * 
     * @param CrudSupplyTek $crudSupplyTek Object that allows to perform queries from datasource
     * @param SessionHandler $sessionHandler Object that allows to manipulate session
     * @param FormatData $formatData Object that allows to format data
     * @param object $notificationService Object to access NotificationService
     * 
     * @return void
     * 
     * @access public
     * @author Rômulo Alves Lousada
     */
    public function __construct(CrudSupplyTek $crudSupplyTek, SessionHandler $sessionHandler, FormatData $formatData, $notificationService) 
    {
        $this->crud = $crudSupplyTek;
        $this->session = $sessionHandler;
        $this->format = $formatData;
        $this->notificationService = $notificationService;
    }
    
    /**
     * Return all branches from an operator
     * 
     * Receives data from request and filters all branches from
     * a specific operator and regions selected on chart.
     * 
     * @param Request $request Object that receives data from the caller
     * @param Response $response Object that sends back data to the caller
     *
     * @return array
     * @throws Exception
     * 
     * @access public
     * @author Rômulo Alves Lousada
     */
    public function getBranchesByOperator(Request $request, Response $response)
    {
        try {
            $params["OPERATOR_ID"] = $this->session->getSessionData('CDOPERADOR');
            
            $filterCriteria = $request->getFilterCriteria();
            $filter = $this->crud->processFilter($filterCriteria);
            
            if (!isset($filter["STATE"])) {
                $params["STATE"] = "T";                
            }
            
            $branches = $this->crud->find($request, $params);
            
            $branches['data'] = $this->formatBranchValues($branches['data']);

            $response->setMetadata($branches);
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    /**
     * Format the values returned from Database
     * 
     * Searches the entire array of data returned from the database,
     * validating and formmating the values of the phone number and
     * responsible.
     * 
     * @param array $data Array with data returned from the database
     *
     * @return array
     * 
     * @access private
     * @author Rômulo Alves Lousada
     */
    private function formatBranchValues($data)
    {
        foreach ($data as $index => $value) {
            $phone = $data[$index]["PHONE"];
            $data[$index]["PHONE"] = $this->format->formatPhone($phone);
            
            $responsible = $data[$index]["RESPONSIBLE"];
            $data[$index]["RESPONSIBLE"] = $this->format->capitalizeWords($responsible);
            
            $street = $data[$index]["STREET"];
            $neighborhood = $data[$index]["NEIGHBORHOOD"];
            $county = $data[$index]["COUNTY"];
            $data[$index]["ADDRESS"] = $this->format->formatAddress($street, $neighborhood, $county);
        }
        
        return $data;
    }
    
    /**
     * Save notifications sent via Control Panel
     * 
     * Receives a notification text that must be sent to all users related
     * to the selected unities.
     * 
     * @param Request $request Object that receives data from the caller
     * @param Response $response Object that sends back data to the caller
     *
     * @return array
     * @throws Exception
     * 
     * @access public
     * @author Rômulo Alves Lousada
     */
    public function sendMessageToBranches(Request $request, Response $response)
    {
        try {
            $rows = $request->getDataSet()->getRows();
            
            if (isset($rows[0]["formData"]["branches"])) {
                $branches = $rows[0]["formData"]["branches"];
                $params["UNIT_ID"] = is_array($branches) ? implode($branches, "_") : $branches;
                
                $data = $this->crud->executeQuery("send-notice", $params);
                
                if (sizeof($data) > 0) {
                    foreach ($data as $notification) {
                        $this->getComplemetaryData($notification, $rows);
                        $this->notificationService->insertNotification($notification);
                    }
                    
    
                    $response->setMetadata(array("OK"));
                }
            } else {
                throw new \Exception("Erro ao tentar selecionar as unidades.", 406);
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    /**
     * Get complementary data needed to save on notifications table.
     * 
     * Brings data from session that are needed to be saved on database,
     * and creates more positions on the array that will be sent to the service,
     * containing all data needed to make the insert statement.
     * 
     * @param array &$notification Array with data from notification that will have more data added to it.
     * @param array $rows Array with data received from the POST request.
     *
     * @return void
     * 
     * @access private
     * @author Rômulo Alves Lousada
     */
    private function getComplemetaryData(&$notification, $rows)
    {
        $operatorId = $this->session->getSessionData("CDOPERADOR");
        $operatorName = $this->session->getSessionData("NMOPERADOR");
        
        $notification["NOTIFICATION_NAME"] = "Aviso de " . $operatorName;
        $notification["NOTIFICATION_TEXT"] = $rows[0]["formData"]["notice"];
        $notification["LOGGED_OPERATOR_ID"] = $operatorId;
    }
    
    /**
     * Update currently logged branch.
     * 
     * Changes the current branch that the user is currently viewing the data.
     * 
     * @param Request $request Object that receives data from the caller
     * @param Response $response Object that sends back data to the caller
     *
     * @return array
     * 
     * @access public
     * @author Rômulo Alves Lousada
     */
    public function updateLoggedBranch(Request $request, Response $response)
    {
        try {
            $rows = $request->getDataSet()->getRows();
            
            if (isset($rows[0]["branch"]["ID"])) {
                $this->session->setSessionData("CDFILIAL", $rows[0]["branch"]["ID"]);
            }
            
            $response->setMetadata(array("OK"));
        } catch (\Exception $e) {
            throw new \Exception ($e->getMessage(), $e->getCode());
        }
    }
}