<?php
namespace SupplyTek\Controllers;

use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use SupplyTek\Utilities\SessionHandler;
use SupplyTek\Utilities\CrudSupplyTek;
use SupplyTek\Services\CartService;
use SupplyTek\Controllers\ApprovalWorkflowController;

/**
 * Class that handle process of cart
 * 
 * @author Danilo D. Vaz
 */
class CartController 
{
    protected $crud;
    protected $session;
    protected $cartService;
    protected $approvalWorkflow;

    /**
     * Class constructor
     * 
     * Initialize an object's properties upon creation of the object.
     * 
     * @param CrudSupplyTek $crudSupplyTek Object that allows to perform queries from datasource
     * @param SessionHandler $sessionHandler Object that allows to manipulate session
     * @param CartService $cartService Object that allows to perform insert, update and delete
     * @param ApprovalWorkflowController $approvalWorkflow Object that start workflow process
     * 
     * @return void
     * 
     * @access public
     * @author Danilo D. Vaz
     */
    public function __construct (CrudSupplyTek $crudSupplyTek, SessionHandler $sessionHandler, CartService $cartService, ApprovalWorkflowController $approvalWorkflow)
    {
        $this->crud = $crudSupplyTek;
        $this->session = $sessionHandler;
        $this->cartService = $cartService;
        $this->approvalWorkflow = $approvalWorkflow;
    }
    
    /**
     * Return all carts from an operator
     * 
     * Filters all carts from operator and unit logged.
     * 
     * @param Request $request Object that receives data from the caller
     * @param Response $response Object that sends back data to the caller
     *
     * @return array
     * @throws Exception
     * 
     * @access public
     * @author Danilo D. Vaz
     */
    public function getCart (Request $request, Response $response)
    {
        try {
            $params['OPERATOR_ID'] = $this->session->getSessionData('CDOPERADOR');
            $params['UNIT_ID'] = $this->session->getSessionData('CDFILIAL');
            $params['PURCHASE_SITUATION'] = 'A';
            
            $cart = $this->crud->find($request, $params);
                
            $response->setMetadata($cart);
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage(), 500);
        }
    }
    
    /**
     * Delete a cart and it’s items
     * 
     * Check if cart exist for that operator and unit and delete the cart and
     * items from cart.
     * 
     * @param Request $request Object that receives data from the caller
     * @param Response $response Object that sends back data to the caller
     *
     * @return void
     * @throws Exception
     * 
     * @access public
     * @author Danilo D. Vaz
     */
    public function deleteCart (Request $request, Response $response)
    {
        $parameters = $request->getParameters();
        $cartId = isset($parameters['CART_ID']) ? $parameters['CART_ID'] : false;
        
        try {
            $this->cartService->beginTransaction();
            
            if ($cartId) {
                $operatorId = $this->session->getSessionData('CDOPERADOR');
                $unitId = $this->session->getSessionData('CDFILIAL');
                $purchaseSituation = 'A';
                
                $this->checkCartExist($operatorId, $unitId, $cartId, $purchaseSituation);
                
                $this->cartService->deleteItemCart($cartId);
                $this->cartService->deleteCartWithoutItem($cartId);
                
                $this->cartService->commit();
            } else {
                throw new \Exception('Parameters not found');
            }
        } catch (\Exception $e) {
            $this->cartService->rollback();
            throw new \Exception($e->getMessage(), 500);
        }
    }
    
    /**
     * Return all items from cart
     * 
     * Receives cart id from request and filters all items from
     * that specific cart
     * 
     * @param Request $request Object that receives data from the caller
     * @param Response $response Object that sends back data to the caller
     *
     * @return array
     * @throws Exception
     * 
     * @access public
     * @author Danilo D. Vaz
     */
    public function getCartItems (Request $request, Response $response)
    {
        try {
            $params['OPERATOR_ID'] = $this->session->getSessionData('CDOPERADOR');
            $params['UNIT_ID'] = $this->session->getSessionData('CDFILIAL');
            $params['PURCHASE_SITUATION'] = 'A';
            
            $cartItems = $this->crud->find($request, $params);
                
            $response->setMetadata($cartItems);
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage(), 500);
        }
    }
    
    /**
     * Delete an item from cart
     * 
     * Check if item exist for that operator and unit and delete the item.
     * If there is no more items in the cart, the cart is deleted.
     * 
     * @param Request $request Object that receives data from the caller
     * @param Response $response Object that sends back data to the caller
     *
     * @return void
     * @throws Exception
     * 
     * @access public
     * @author Danilo D. Vaz
     */
    public function deleteCartItem (Request $request, Response $response)
    {
        $parameters = $request->getParameters();
        $cartItemId = isset($parameters['CART_ITEM_ID']) ? $parameters['CART_ITEM_ID'] : false;
        
        try {
            $this->cartService->beginTransaction();
            
            if ($cartItemId) {
                $operatorId = $this->session->getSessionData('CDOPERADOR');
                $unitId = $this->session->getSessionData('CDFILIAL');
                $purchaseSituation = 'A';
                
                $cartItem = $this->checkCartItemExist($operatorId, $unitId, $cartItemId, $purchaseSituation);
                
                $this->cartService->deleteItemCart($cartItem['CART_ID'], $cartItemId);
                $this->cartService->deleteCartWithoutItem($cartItem['CART_ID']);
                
                $this->cartService->commit();
            } else {
                throw new \Exception('Parameters not found');
            }
        } catch (\Exception $e) {
            $this->cartService->rollback();
            throw new \Exception($e->getMessage(), 500);
        }
    }
    
    /**
     * Update an item from cart
     * 
     * Check if item exist for that operator and unit and update the item.
     * If the amount is 0, the item will be deleted.
     * 
     * @param Request $request Object that receives data from the caller
     * @param Response $response Object that sends back data to the caller
     *
     * @return void
     * @throws Exception
     * 
     * @access public
     * @author Danilo D. Vaz
     */
    public function updateCartItem (Request $request, Response $response)
    {
        $rows = $request->getDataSet()->getRows();
        $parameters = $request->getParameters();
        $cartItem = reset($rows);
        $cartItemId = isset($parameters['CART_ITEM_ID']) ? $parameters['CART_ITEM_ID'] : false;
        
        try {
            $this->cartService->beginTransaction();
            
            if ($cartItemId && $cartItem) {
                $operatorId = $this->session->getSessionData('CDOPERADOR');
                $unitId = $this->session->getSessionData('CDFILIAL');
                $purchaseSituation = 'A';
                
                $checkCartItem = $this->checkCartItemExist($operatorId, $unitId, $cartItemId, $purchaseSituation);
                
                if ($cartItem['ITEMS'] > 0) {
                    $this->cartService->updateItemCart($cartItemId, $cartItem['ITEMS']);
                } else {
                    $this->cartService->deleteItemCart($checkCartItem['CART_ID'], $cartItemId);
                    $this->cartService->deleteCartWithoutItem($checkCartItem['CART_ID']);
                }
                
                $this->cartService->commit();
            } else {
                throw new \Exception('Parameters not found');
            }
        } catch (\Exception $e) {
            $this->cartService->rollback();
            throw new \Exception($e->getMessage(), 500);
        }
    }

    /**
     * Add product in cart
     * 
     * Receives the product and validate the infomation. Can insert, update and
     * delete the product from cart, according the parameters.
     * 
     * @param Request $request Object that receives data from the caller
     * @param Response $response Object that sends back data to the caller
     *
     * @return void
     * @throws Exception
     * 
     * @access public
     * @author Danilo D. Vaz
     */
    public function addItemCart (Request $request, Response $response)
    {
        try {
            $product = $request->getDataSet()->getRows();
            
            $this->cartService->beginTransaction();
            
            if (count($product)) {
                $product = reset($product);
                $product['SIZE'] = isset($product['SIZE']) && $product['SIZE'] != '' ? $product['SIZE'] : 'UNICO';
                
                $operatorId = $this->session->getSessionData('CDOPERADOR');
                $unitId = $this->session->getSessionData('CDFILIAL');
                $orgId = $this->session->getSessionData('NRORG');
                
                $this->checkProductCatalogExist($product, $operatorId, $unitId);
                
                $cartExist = $this->checkCartExistProduct($operatorId, $unitId, $product['ID']);
                
                if ($product['AMOUNT'] && $product['PRICE']) {
                    if ($cartExist) {
                        $cartId = $cartExist['NRCARTSUPPLY'];
                        $itemExistCart = $this->checkProductExistCart($cartId, $operatorId, $unitId, $product);

                        if ($itemExistCart) {
                            $amount = $product['AMOUNT'] + $itemExistCart['QTPRODUTO'];
                            $this->cartService->updateItemCart($itemExistCart['NRPRODCART'], $amount);
                        } else {
                            $this->cartService->insertItemCart($cartId, $operatorId, $unitId, $product, $orgId);
                        }
                    } else {
                        $cartId = $this->cartService->insertCart($operatorId, $unitId, $product['ID'], $orgId);
                        $this->cartService->insertItemCart($cartId, $operatorId, $unitId, $product, $orgId);
                    }
                } else {
                    if ($cartExist) {
                        $cartId = $cartExist['NRCARTSUPPLY'];
                        $itemExistCart = $this->checkProductExistCart($cartId, $operatorId, $unitId, $product);
                        
                        if ($itemExistCart) {
                            $this->cartService->deleteItemCart($cartId, $itemExistCart['NRPRODCART']);
                            $this->cartService->deleteCartWithoutItem($cartId);
                        }
                    }
                }
                
                $this->cartService->commit();
            } else {
                throw new \Exception('Parameters not found');
            }
        } catch (\Exception $e) {
            $this->cartService->rollback();
            throw new \Exception($e->getMessage(), 500);
        }
    }
    
    /**
     * Send cart to order
     * 
     * Receives the information and run process and validation to create supply
     * authorization. Start the approval workflow.
     * 
     * @param Request $request Object that receives data from the caller
     * @param Response $response Object that sends back data to the caller
     *
     * @return void
     * @throws Exception
     * 
     * @access public
     * @author Danilo D. Vaz
     */
    public function sendCartOrder (Request $request, Response $response)
    {
        $cartOrder = $request->getDataSet()->getRows();
        
        $this->cartService->beginTransaction();
        
        try {
            if (count($cartOrder)) {
                $supplierId = $cartOrder[0]['order']['SUPPLIER_ID'];
                $cartId = $cartOrder[0]['order']['ID'];
                $deliveryDate = $cartOrder[0]['formData']['deliveryDate'];
                $comment = $cartOrder[0]['formData']['commentText'];
                $firstApprover = $cartOrder[0]['formData']['selectApprover1'];
                $operatorId = $this->session->getSessionData('CDOPERADOR');
                $unitId = $this->session->getSessionData('CDFILIAL');
                $orgId = $this->session->getSessionData('NRORG');
                
                $apurationItem = $this->getApurationItem($operatorId, $unitId, $cartId);
                $purchaseCenter = $apurationItem['CDFILCENTCMP'];
                $purchaser = $apurationItem['CDCOMPRADOR'];
                
                $apurationId = $this->apurationOrder($cartId, $purchaseCenter, $unitId, $purchaser, $deliveryDate, $orgId);
                $this->processSupplyAuth($cartId, $apurationId, $operatorId, $comment, $firstApprover, $supplierId);
                $this->cartService->updateCart($cartId, 'E');
                
                $this->cartService->commit();
            } else {
                throw new \Exception('Parameters not found');
            }
        } catch (\Exception $e) {
            $this->cartService->rollback();
            throw new \Exception($e->getMessage(), 500);
        }
    }
    
    /**
     * Run order apuration
     * 
     * Receives the information and run process and validation to apuration 
     * order and generate supply authorization
     * 
     * @param string $cartId String that receives cart id
     * @param string $purchaseCenter String that receives purchase center
     * @param string $unitId String that receives unit id
     * @param string $purchaser String that receives purchaser
     * @param string $deliveryDate String that receives delivery date
     * @param string $orgId String that receives organization id
     *
     * @return string
     * 
     * @access private
     * @author Danilo D. Vaz
     */
    private function apurationOrder ($cartId, $purchaseCenter, $unitId, $purchaser, $deliveryDate, $orgId)
    {
        $date = date('d/m/Y');
        $deliveryDate = \DateTime::createFromFormat('Y-m-d', $deliveryDate)->format('d/m/Y');

        $apurationId = $this->cartService->execProcApuracaoSupplytek($purchaseCenter, $deliveryDate, $cartId, $unitId, $orgId);
        $this->cartService->execProcEfetivarAf($purchaser, $date, $purchaseCenter, $apurationId, $unitId);
        
        return $apurationId;
    }
    
    /**
     * Get apuration item
     * 
     * Check if exist for that unit and operator and returns apuration item
     * 
     * @param string $operatorId String that receives operator id
     * @param string $unitId String that receives unit id
     * @param string $cartId String that receives cart id
     *
     * @return array
     * @throws Exception
     * 
     * @access private
     * @author Danilo D. Vaz
     */
    private function getApurationItem ($operatorId, $unitId, $cartId)
    {
        $param = array(
            "OPERATOR_ID" => $operatorId,
            "UNIT_ID" => $unitId,
            "CART_ID" => $cartId,
            "PURCHASE_SITUATION" => 'A'
        );
        
        $item = $this->crud->executeQuery('apuration-item', $param);
        
        if (!count($item)) {
            throw new \Exception('Apuration item not found');
        }
        
        return reset($item);
    }
    
    /**
     * Run process in supply authorization
     * 
     * Get supply authorization generated in apuration and run process for each
     * 
     * @param string $cartId String that receives cart id
     * @param string $apurationId String that receives apuration id
     * @param string $operatorId String that receives operator id
     * @param string $comment String that receives justification for approval
     * @param string $firstApprover String that receives the first approver of workflow
     * @param string $supplierId String that receives the bill's supplier
     *
     * @return void
     * 
     * @access private
     * @author Danilo D. Vaz
     */
    private function processSupplyAuth ($cartId, $apurationId, $operatorId, $comment, $firstApprover, $supplierId)
    {
        $listSupplyAuth = $this->getSupplyAuthorization($apurationId);
        $result = $this->cartService->verifySupplierGenerateBillToPay($supplierId);
        
        foreach ($listSupplyAuth as $supplyAuth) {
            $unitDest = $supplyAuth['CDFILDEST'];
            $unitEntry = $supplyAuth['CDFILLANC'];
            $supplyAuthId = $supplyAuth['NRAUTOFORN'];
            
            $this->cartService->execProcFechamentoAF($unitDest, $unitEntry, $supplyAuthId);
            $this->cartService->updateSupplyAuth($unitDest, $unitEntry, $supplyAuthId, $operatorId, $comment, $cartId);
            $this->cartService->updateSupplyAuthItem($unitDest, $unitEntry, $supplyAuthId);
            
            if($result['IDGERCONTPGT'] == "S"){
                $this->cartService->execProcExcluiContaPagarPrevista($unitDest, $unitEntry, $supplyAuthId);
                $this->cartService->execProcGeraContaPagarPrevista($unitDest, $unitEntry, $supplyAuthId);
            }
            
            $paramWorkflow = array(
                "P_CDFILDEST" => $unitDest,
                "P_CDFILLANC" => $unitEntry,
                "P_NRDEMANDA" => $supplyAuthId,
                "P_CDOPERADOR" => $firstApprover
            );

            $this->approvalWorkflow->startWorkflow($paramWorkflow);
        }
    }
    
    /**
     * Get supply authorization
     * 
     * Get supply authorization generated in apuration and check if exist
     * 
     * @param string $apurationId String that receives apuration id
     *
     * @return array
     * @throws Exception
     * 
     * @access private
     * @author Danilo D. Vaz
     */
    private function getSupplyAuthorization ($apurationId)
    {
        $param = array(
            "APURATION_ID" => $apurationId
        );
        
        $supplyAuth = $this->crud->executeQuery('supply-authorization-apuration', $param);
        
        if (!count($supplyAuth)) {
            throw new \Exception('Supply authorization not found');
        }
        
        return $supplyAuth;
    }
    
    /**
     * Check if cart exist
     * 
     * Check if cart exist for that operator and unit and returns it
     * 
     * @param string $operatorId String that receives operator id
     * @param string $unitId String that receives unit id
     * @param string $cartId String that receives cart id
     * @param string $purchaseSituation String that receives purchase situation
     *
     * @return array
     * @throws Exception
     * 
     * @access private
     * @author Danilo D. Vaz
     */
    private function checkCartExist ($operatorId, $unitId, $cartId, $purchaseSituation)
    {
        $param = array(
            "OPERATOR_ID" => $operatorId,
            "UNIT_ID" => $unitId,
            "CART_ID" => $cartId,
            "PURCHASE_SITUATION" => $purchaseSituation
        );
        
        $cart = $this->crud->executeQuery('check-cart-exist', $param);
        
        if (!count($cart)) {
            throw new \Exception('Cart not found');
        }
        
        return reset($cart);
    }
    
    /**
     * Check if cart item exist
     * 
     * Check if cart item exist for that operator and unit and returns it
     * 
     * @param string $operatorId String that receives operator id
     * @param string $unitId String that receives unit id
     * @param string $cartItemId String that receives cart item id
     * @param string $purchaseSituation String that receives purchase situation
     *
     * @return array
     * @throws Exception
     * 
     * @access private
     * @author Danilo D. Vaz
     */
    private function checkCartItemExist ($operatorId, $unitId, $cartItemId, $purchaseSituation)
    {
        $param = array(
            "OPERATOR_ID" => $operatorId,
            "UNIT_ID" => $unitId,
            "CART_ITEM_ID" => $cartItemId,
            "PURCHASE_SITUATION" => $purchaseSituation
        );
        
        $cartItem = $this->crud->executeQuery('check-cart-item-exist', $param);
        
        if (!count($cartItem)) {
            throw new \Exception('Cart item not found');
        }
        
        return reset($cartItem);
    }
    
    /**
     * Checks if product received exist in catalog
     * 
     * Search for supplied product for that operator and unit and checks if 
     * it exists and if only one returned
     * 
     * @param array $product Array with product information provider by frontend
     * @param string $operatorId String with operator id
     * @param string $unitId String with unit id
     *
     * @return void
     * @throws Exception
     * 
     * @access private
     * @author Danilo D. Vaz
     */
    private function checkProductCatalogExist ($product, $operatorId, $unitId)
    {
        $params = array(
            "ID" => $product['ID'],
            "PRICE" => $product['PRICE'],
            "SIZES" => $product['SIZE'] ? $product['SIZE'] : "UNICO",
            "UNIT_ID" => $unitId,
            "OPERATOR_ID" => $operatorId
        );
        
        $prodCatalog = $this->crud->executeQuery('check-prodcatalog-exist', $params);
        
        $prodCatalogExist = count($prodCatalog);

        if ($prodCatalogExist) {
            if ($prodCatalogExist > 1) {
                throw new \Exception('Found more than one product');
            }
        } else {
            throw new \Exception('Product not found in catalog');
        }
    }
    
    /**
     * Checks if cart exist for that product
     * 
     * Search for cart for that operator and unit and checks if it exists and 
     * if only one returned
     * 
     * @param string $operatorId String with operator id
     * @param string $unitId String with unit id
     * @param string $productId String with product id
     *
     * @return array
     * @throws Exception
     * 
     * @access private
     * @author Danilo D. Vaz
     */
    private function checkCartExistProduct ($operatorId, $unitId, $productId)
    {
        $params = array(
            "OPERATOR_ID" => $operatorId,
            "UNIT_ID" => $unitId,
            "ID" => $productId
        );
        
        $cart = $this->crud->executeQuery('check-cart-exist-product', $params);
        
        $cartExist = count($cart);
        
        if ($cartExist) {
            if ($cartExist == 1) {
                $cart = reset($cart);
            } else {
                throw new \Exception('Found more than one cart');
            }
        } else {
            $cart = false;
        }
        
        return $cart;
    }
    
    /**
     * Checks if product exist in cart
     * 
     * Search for the product in cart for that operator and unit and checks if 
     * it exists and if only one returned
     * 
     * @param string $cartId String with cart id
     * @param string $operatorId String with operator id
     * @param string $unitId String with unit id
     * @param array $product Array with product information
     *
     * @return array
     * @throws Exception
     * 
     * @access private
     * @author Danilo D. Vaz
     */
    private function checkProductExistCart ($cartId, $operatorId, $unitId, $product)
    {
        $params = array(
            "ID" => $product['ID'],
            "SIZES" => $product['SIZE'],
            "PRICE" => $product['PRICE'],
            "NRCARTSUPPLY" => $cartId,
            "OPERATOR_ID" => $operatorId,
            "UNIT_ID" => $unitId
        );
        
        $itemCart = $this->crud->executeQuery('check-product-exist-cart', $params);
        
        $itemExistCart = count($itemCart);
        
        if ($itemExistCart) {
            if ($itemExistCart == 1) {
                $itemCart = reset($itemCart);
            } else {
                throw new \Exception('Found more than one of the same product in cart');
            }
        } else {
            $itemCart = false;
        }
        
        return $itemCart;
    }
    
    public function getDeliveryDate (Request $request, Response $response)
    {
        try {
            $row = $request->getDataSet()->getRows();
            $supplier = $row[0]["row"][0]["value"];
            $deliveryDate = $row[0]["row"][1]["value"];
            $formattedDeliveryDate = date('d/m/Y', strtotime($deliveryDate));
            $unitId = $this->session->getSessionData('CDFILIAL');
            $date = $this->cartService->validateDeliveryDate($unitId, $supplier, $formattedDeliveryDate);
            $nextAvailableDate = $this->cartService->getDeliveryDate($unitId, $supplier);
            $deliveryDate = array (
                "deliveryDate" => $date,
                "nextAvailableDate" => $nextAvailableDate
            );
            
            $response->setMetadata($deliveryDate);
        } catch(\Exception $e) {
            throw new \Exception ($e->getMessage(), $e->getCode());
        }
    }
        
}
