<?php
namespace SupplyTek\Controllers;

use Zeedhi\Framework\DTO;
use SupplyTek\Utilities\CrudSupplyTek;
use SupplyTek\Utilities\SessionHandler;

/**
 * Search for suppliers information.
 *
 * @author Bruna Chaves Carneiro
 */
class SuppliersController 
{
    protected $crud;
    protected $sessionHandler;
    protected $fileServerURL;

    public function __construct (CrudSupplyTek $crudSupplyTek, SessionHandler $sessionHandler, $fileServerURL) 
    {
        $this->crud = $crudSupplyTek;
        $this->sessionHandler = $sessionHandler;
        $this->fileServerURL = $fileServerURL;
    }

    /**
     * Returns the catalog suppliers that supply the unit connected to the system
     * 
     * Checks whether the parameter is being passed through the route; in this 
     * case, the parameter 'ID' will receive the value of the parameter 
     * found returning a single supplier; otherwise, it will use 'T' to return 
     * all suppliers found.
     *
     * @param Request $request – Object that receives data from the caller
     * @param Response $response –  Object that sends back data to the caller
     * 
     * @return array
     * @throws Exception
     * 
     * @access public
     * @author Bruna Chaves Carneiro
     */
    public function getSuppliers(DTO\Request $request, DTO\Response $response)
    {
        try {
            $filterUri = $request->getParameters();
            $cdFilial = $this->sessionHandler->getSessionData('CDFILIAL');
            $nrOrg = $this->sessionHandler->getSessionData('NRORG');
            
            $params = array(
                "P_CDFILIAL"   => $cdFilial,
                "P_NRORG"      => $nrOrg
            );
            
            if(empty($filterUri)) {
                $params['ID'] = 'T';
            }
            
            $suppliers = $this->crud->find($request, $params);
        
            $newDataSet = $this->setImgFullPath($suppliers);
    
            $response->setMetadata($newDataSet);
        } catch (\Exception $e) {
            throw $e;
        }
    }
	
    /**
     * Returns correctly handled vendor information for display
     * 
     * Obtain the link of the image of the supplier's logo on the file server and 
     * link it with the link of the image brought from the bank so that the image 
     * is displayed on the system.
     *
     * @param string $suppliers – Array with the main suppliers of the catalog
     * 
     * @return array
     * 
     * @access private
     * @author Bruna Chaves Carneiro
     */
	private function setImgFullPath($suppliers) 
	{
        if(!empty($suppliers['data'])) {
    	   if (array_key_exists(0, $suppliers['data'])) {
        		foreach ($suppliers['data'] as &$supplier) {
        			if ($supplier['IMAGE']) {
        				$supplier['IMAGE'] = $this->fileServerURL . $supplier['IMAGE'];
        			} 
        		}
            } else {
                if($suppliers['data']['IMAGE']) {
                    $suppliers['data']['IMAGE'] = $this->fileServerURL . $suppliers['data']['IMAGE'];
                }
            }
        }
		return $suppliers;
	}
	
    /**
     * Returns the main suppliers of the catalog
     * 
     * Returns the catalog suppliers that supply the unit connected to the system
     * that has the most orders and the information regarding the last order of 
     * each supplier.
     *
     * @param Request $request – Object that receives data from the caller.
     * @param Response $response –  Object that sends back data to the caller
     * 
     * @return array
     * @throws Exception
     * 
     * @acess public
     * @author Bruna Chaves Carneiro
     */
    public function getTopSuppliers(DTO\Request $request, DTO\Response $response)
    {
        try {
            $cdFilial = $this->sessionHandler->getSessionData('CDFILIAL');
            $nrOrg = $this->sessionHandler->getSessionData('NRORG');
            $params = array(
                "P_CDFILIAL"   => $cdFilial,
                "P_NRORG"      => $nrOrg
            );
            $suppliers = $this->crud->executeQuery('top-suppliers-order', $params);
            $cdSuppliers = [];
            
            foreach($suppliers as $supplier) {
                array_push($cdSuppliers, $supplier['CDFORNECED']);
            }
            
            $paramsFind = array(
                "P_CDFILIAL"   => $cdFilial,
                "ID" => join('_', $cdSuppliers)
            );
            
            $supplierInfoOrder = $this->crud->find($request, $paramsFind);
            
            $response->setMetadata($supplierInfoOrder);
        } catch (\Exception $e) {
            throw $e;
        }
    }

}